% Example for Case Study III - Stability analysis of tumor dormancy

% Parameters definition
q=10;
r=0.9;
%alph=0.3;
alph=0.08;
k1=0.8;
bet=0.1;
d1=0.02;
s=0.8;
k2=0.7;
d2=0.03;

% Matrices of the system in the original coordinate system
A=diag([r,-d1,s-d2]);

nstates=length(A);

B1=[-r/k1,  -alph/2,    0;
    -alph/2,    0,      0;
        0,      0,      0];    

B2=[0   0   0;
    0   0   bet/2;
    0   bet/2   0];

B3=[0   0   0;
    0   0   -bet/2;
    0   -bet/2  -s/k2];


% Define initial condition
x0=[2.5; 1.5; 0.5];

% Define the box around E3
%   2 <= x1 <= 3.5
%   1 <= x2 <= 6
% 0.1 <= x3 <= 0.7;
% Rx=[    2   3.5;
%         1     6;
%       0.1   0.7 ];

Rx=[    2     7;
    5.2     5.7;
    0.19   0.21 ];


% This box has to be translated to the equilibrium point E3
E3=[(-(alph*s/bet*(1-d1/bet/k2)-alph*d2/bet-r)+...
    sqrt((alph*s/bet*(1-d1/bet/k2)-alph*d2/bet-r)^2+4*r*q/k1))/(2*r/k1);
    s/bet*(1-d1/bet/k2)-d2/bet;
    d1/bet]
%E3=[2.67; 5.41; 0.2];

% Find the equilibrium points
syms x1 x2 x3
x=[x1;x2;x3];
fx=A*x+[x.'*B1; x.'*B2; x.'*B3]*x+[q;0;0];
xes=solve(fx);

% Compute stability of equilibrium points
for i=1:length(xes.x1)
    Xe(:,i)=double([xes.x1(i);xes.x2(i);xes.x3(i)]);
    display(['Eq. point ' num2str(i) ': ' num2str(Xe(:,i)')])    
    display(['Eigenvalues of eq. point ' num2str(i) ': '...
    num2str(eig(A+2*[Xe(:,i).'*B1;Xe(:,i).'*B2;Xe(:,i).'*B3])')])
end

pause

% The translated box is defined by
% -0.67 <= x1 <= 0.83;
% -4.41 <= x2 <= 0.59;
% -0.1  <= x3 <= 0.5;
Rxt=[Rx(:,1)-E3, Rx(:,2)-E3];


%Vertices of the polytope
Xvert=[];
for i1=1:2
    for i2=1:2
        for i3=1:2
            Xvert=[Xvert, [Rxt(1,i1);Rxt(2,i2);Rxt(3,i3)]];
        end
    end
end

Ak=[1/Rxt(1,1)      0          0;
    1/Rxt(1,2)      0          0;
    0           1/Rxt(2,1)     0;
    0           1/Rxt(2,2)     0;
    0               0        1/Rxt(3,1);
    0               0        1/Rxt(3,2)   ]';
% 
% Xvert=Xvert/15;
% Ak=Ak*15;


% The change of variables has to be applied also to the system
% The new A matrix reads
Ah=A+2*[E3'*B1;E3'*B2;E3'*B3];

% If \gamma is defined as an optimization variable, the problem is bilinear
% and we cannot use LMILAB. Instead a linear search over gam is performed
%gam=sdpvar(1,1);
gam_v=logspace(-3,0,40);
P=sdpvar(size(A,1));

for i=length(gam_v):-1:1
    gam=gam_v(i);
    % Inequalities definition
    %F=[0<gam<1];
    F=set([]);
    F=F+set(P>0);
    for k=1:2*length(A)    
        F=F+set([1, gam*Ak(:,k)'; gam*Ak(:,k),P]>=0);
    end
    for k=1:2^length(A)
        F=F+set(Xvert(:,k)'*P*Xvert(:,k)<=1);
        F=F+set(gam*(Ah'*P+P*Ah)+P*[Xvert(:,k)'*B1;Xvert(:,k)'*B2;Xvert(:,k)'*B3]+[Xvert(:,k)'*B1;Xvert(:,k)'*B2;Xvert(:,k)'*B3]'*P<0);
    end
    
    solvesdp(F,[]);
    
    [primalfeas,dualfeas] = checkset(F);
    
    if (primalfeas>0)
        break
    end
end

% Draw 3d plot with the polytope included into the invariant ellipsoid
alpha_transparency=0.25;
% Number of discrete elements of the surface
Nsurf=100;

figure(1)
axes('CameraPosition',[54.1524   -3.1284    1.0857],...
    'CameraTarget',[3,5.6,0.225])
grid on
hold on
plot3(E3(1),E3(2),E3(3),'Marker','.','MarkerSize',24)
eigP=eig(double(P));

[x,y,z]=ellipsoid(E3(1),E3(2),E3(3),1/sqrt(eigP(1)),...
    1/sqrt(eigP(2)),1/sqrt(eigP(3)),Nsurf);
ellipsoid_h=surfl(x,y,z)%,'light')
shading interp
set(ellipsoid_h,'EdgeColor','none','FaceColor',0.8*ones(1,3),...
    'SpecularStrength',0.6,'DiffuseStrength',0.3);
% Set transparency
alpha(alpha_transparency);
% Translate the box from the origin to the equilibrium point
Xvert_E3box(1,:)=Xvert(1,:)+E3(1);
Xvert_E3box(2,:)=Xvert(2,:)+E3(2);
Xvert_E3box(3,:)=Xvert(3,:)+E3(3);
% Draw the box
box_faces=[1,5,6,2;
    1,2,4,3;
    5,6,8,7;
    3,7,8,4;
    1,5,7,3;
    2,6,8,4];
patch('Vertices',Xvert_E3box','Faces',box_faces,'FaceColor',0.9*ones(1,3))
% Set transparency
alpha(alpha_transparency);
light('Position',[20,6,0.5])
xlabel('M');ylabel('N');zlabel('Z')

% Simulation and plot with initial condition at a vertex
figure(2)
subplot(3,1,1)
hold on
ylabel('M')
subplot(3,1,2)
hold on
ylabel('N')
subplot(3,1,3)
hold on
ylabel('Z')
xlabel('Time')

Nplots=6;
for i=1:Nplots
    [tout,yout]=ode23t(@quadr_sys,[0,70],Rx(:,1)*(i-1)/(Nplots-1)...
        +(1-(i-1)/(Nplots-1))*Rx(:,2),[],alph);    
    subplot(3,1,1)
    plot(tout,yout(:,1))
    subplot(3,1,2)
    plot(tout,yout(:,2))
    subplot(3,1,3)
    plot(tout,yout(:,3))
    figure(1)
    plot3(yout(1,1),yout(1,2),yout(1,3),'Marker','x','MarkerSize',10,...
        'LineWidth',2);
    plot3(yout(:,1),yout(:,2),yout(:,3))
    figure(2)
end

subplot(3,1,1)
fig_ax=axis;
axis([0,2,fig_ax(3),fig_ax(4)])
ylabel('M')
subplot(3,1,2)
fig_ax=axis;
axis([0,70,fig_ax(3),fig_ax(4)])
hold on
ylabel('N')
subplot(3,1,3)
fig_ax=axis;
axis([0,70,fig_ax(3),fig_ax(4)])
hold on
ylabel('Z')
xlabel('Time')


% % Simulation and plot with initial condition at another vertex
% [tout,yout]=ode23t(@quadr_sys,[0,70],[Rx(1,1);Rx(2,1);Rx(3,1)]);
% subplot(3,1,1)
% plot(tout,yout(:,1),'--')
% subplot(3,1,2)
% plot(tout,yout(:,2),'--')
% subplot(3,1,3)
% plot(tout,yout(:,3),'--')
% 
% % Simulation and plot with initial condition at another vertex
% [tout,yout]=ode23t(@quadr_sys,[0,70],[Rx(1,1);Rx(2,2);Rx(3,2)]);
% subplot(3,1,1)
% plot(tout,yout(:,1),'-.')
% subplot(3,1,2)
% plot(tout,yout(:,2),'-.')
% subplot(3,1,3)
% plot(tout,yout(:,3),'-.')


% check feasibility
%gam=0.0774;
%
% P=[0.0221, 0.0060, -0.0690;
%    0.0060, 0.0092, -0.0154;
%    -0.0690, -0.0154, 1.1850];
% 
% if any(not(eig(P)>0))
%     display('Infeasible P')
% end
% for k=1:2*length(A)
%     if any(not(eig([1, gam*Ak(:,k)'; gam*Ak(:,k),P])>=0))
%         display('Infeasible cond1')
%         break
%     end
% end
% for k=1:2^length(A)
%     if any(not(eig(Xvert(:,k)'*P*Xvert(:,k))<=1))
%         display('Infeasible cond2')
%         break
%     end
%     if any(not(eig(gam*(Ah'*P+P*Ah)+P*[Xvert(:,k)'*B1;Xvert(:,k)'*B2;Xvert(:,k)'*B3]+[Xvert(:,k)'*B1;Xvert(:,k)'*B2;Xvert(:,k)'*B3]'*P)<0))
%         display('Infeasible cond3')
%         break
%     end
% end