% Simulation of the bacterial chemotaxis model given in
% Barkai, Leibler, Nature, 1997.
clear all

% Simulation time [s]
Tf=180*60;
% Number of simulation points
N_sim_pts=10001;
% Number of methylation sites
n_methyl=2;
% Cell volume [l]
CellVol=1.4e-15;

% Avogadro number
N_Av=6.02214179e23;

% Initial conditions definition
% Methylation and demethylation enzymes concentration [uM]
cheB_in=2000/CellVol/N_Av*1e6;
cheR_in=200/CellVol/N_Av*1e6;
% Ligand concentration [uM]
L_in=1;
% Unoccupied/occupied unmethylated/methylated 
% receptors complex concentrations [uM]
Etot=10000/CellVol/N_Av*1e6;

% Initial guess
Eu0_in=5000/CellVol/N_Av*1e6;
Eo0_in=5000/CellVol/N_Av*1e6;
Eu1_in=0/CellVol/N_Av*1e6;
Eo1_in=0/CellVol/N_Av*1e6;
Eu2_in=0/CellVol/N_Av*1e6;
Eo2_in=0/CellVol/N_Av*1e6;

% % Steady-state values for L_in=1e-5 M;
% Eu0_in=3.5096e-008;
% Eo0_in=3.5096e-007;
% Eu1_in=3.8571e-007;
% Eo1_in=3.9180e-006;
% Eu2_in=2.7523e-007;
% Eo2_in=2.9237e-006;

% Steady-state values for L_in=0 M;
% Eu0_in=1.9841e-006;
% Eo0_in=0;
% Eu1_in=1.9823e-006;
% Eo1_in=0;
% Eu2_in=3.9610e-007;
% Eo2_in=0;

x0=zeros(8+6*(n_methyl-1),1);
x0(1)=Eu0_in;
x0(2)=Eo0_in;
x0(5)=Eu1_in;
x0(6)=Eo1_in;
x0(11)=Eu2_in;
x0(12)=Eo2_in;

%
%load ss_L0 x_ss
%x0=x_ss;

% Simulation time
tv=linspace(0,Tf,N_sim_pts);
% Define input L(t)
%L=L_in*ones(N_sim_pts,1);
L=zeros(N_sim_pts,1);

%L(ceil(N_sim_pts/3):ceil(N_sim_pts/3*2))=4*L_in;

% Twenty minutes pulses of 1,3,5,7 uM ligand concentration every,
% interleaved by twenty minutes at zero concentration
L(ceil(N_sim_pts/9):ceil(N_sim_pts/9*2))=L_in;
L(ceil(N_sim_pts/9*3):ceil(N_sim_pts/9*4))=3*L_in;
L(ceil(N_sim_pts/9*5):ceil(N_sim_pts/9*6))=5*L_in;
L(ceil(N_sim_pts/9*7):ceil(N_sim_pts/9*8))=7*L_in;

%[tout,xout]=ode23s(@ecoli_chemo,tv,x0,[],n_methyl,tv,L,cheB_in,cheR_in);
[tout,xout]=ode15s(@ecoli_chemo_v2,tv,x0(1:8+6*(n_methyl-1)-1),[],...
    Etot,n_methyl,tv,L,cheB_in,cheR_in);

% Activation probabilities
alph  = [0; 0.1; 0.5; 0.75; 1];
alpho = [0;   0; 0.1;  0.5; 1];

% Activity level of the system for 2 methylation sites
Activ=(alph(2)*xout(:,5)+alpho(2)*xout(:,6)...
    +alph(3)*xout(:,11)+alpho(3)*xout(:,12))*CellVol*N_Av/1e6;

% Change time scale to minutes
tout_min=tout/60;

figure
i=0;
i=i+1;
subplot(3,3,i);
plot(tout_min,Activ);
legend('Activity');
i=i+1;
subplot(3,3,i);
%plot(tout_min,xout(:,1)+xout(:,5)+xout(:,11));
%legend('Eu_{tot}')
plot(tout_min,xout(:,1)+xout(:,2));
legend('E_{unmeth}')
i=i+1;
subplot(3,3,i);
%plot(tout_min,xout(:,2)+xout(:,6)+xout(:,12));
%legend('Eo_{tot}')
plot(tout_min,xout(:,5)+xout(:,6)+xout(:,11)+xout(:,12));
legend('E_{meth}')
i=i+1;
subplot(3,3,i);
%plot(tout_min,xout(:,[1,5,11]))
%legend('Eu0','Eu1','Eu2')
plot(tout_min,xout(:,1))
legend('Eu0')
i=i+1;
subplot(3,3,i);
plot(tout_min,xout(:,5))
legend('Eu1')
i=i+1;
subplot(3,3,i);
plot(tout_min,xout(:,11))
legend('Eu2')
i=i+1;
subplot(3,3,i);
%plot(tout_min,xout(:,[2,6,12]))
%legend('Eo0','Eo1','Eo2')
plot(tout_min,xout(:,[2]))
legend('Eo0')
i=i+1;
subplot(3,3,i);
plot(tout_min,xout(:,[6]))
legend('Eo1')
i=i+1;
subplot(3,3,i);
plot(tout_min,xout(:,12))
legend('Eo2')
