% Implementation of the bacterial chemotaxis model given in
% Barkai, Leibler, Nature, 1997.
%
% v2: One differential equation has been eliminated by exploiting the
% constraint on the total concentration of receptor, defined by the new
% input argument Etot [M]

function dx = ecoli_chemo_v2(t,x,Etot,n_methyl,tu,uv,cheB,cheR)

% Define input at current time t by interpolation
u=interp1(tu,uv,t);

% State variables definition
% Total number of state variables equals 8+6*(n_methyl-1)
i=1;
    Eu(i)=x(i);
    Eo(i)=x(i+1);
    EuR(i)=x(i+2);
    EoR(i)=x(i+3);

for i=0:n_methyl-2
    Eu(i+2)=x(5+6*i);
    Eo(i+2)=x(5+6*i+1);
    EuR(i+2)=x(5+6*i+2);
    EoR(i+2)=x(5+6*i+3);
    EuB(i+2)=x(5+6*i+4);
    EoB(i+2)=x(5+6*i+5);
end

i=n_methyl-1;

Eu(i+2)=x(5+6*i);
Eo(i+2)=x(5+6*i+1);
EuB(i+2)=x(5+6*i+2);
%EoB(i+2)=x(5+6*i+3);
% Exploit the constraint on total receptors number, equal to Etot, 
% to eliminate one differential equation
EoB(i+2)=Etot-sum(x(1:end-1));



% Parameters definition
kl=1e3;         % 1/(s uM)
kli=1e3;        % 1/s
%Kr=1.25e-6;     % M
%Kb=1.25e-6;     % M
ar=80;        % 1/(s uM)
arp=80;       % 1/(s uM)
ab=800;       % 1/(s uM)
abp=0;          % 1/(s uM)
%abp=ab/10;       % SET THIS TO ZERO TO GET THE PERFECT ADAPTATION SYSTEM
dr=1e2;         % 1/s
kr=0.1;         % 1/s
db=1e3;         % 1/s
kb=0.1;         % 1/s
%kbu=0.1;        % 1/s
%kbo=0.1;        % 1/s
alph  = [0; 0.1; 0.5; 0.75; 1];
alpho = [0;   0; 0.1;  0.5; 1];
%alph  = [0; 0.75; 1];
%alpho = [0; 0.2; 1];

% Model equations definition

% u is the ligand input

% First iteration -> no demethylation terms
% Eu(1) means unoccupied receptor without methylated sited
% Eu(2) has 1 methylated site, and so on...
i=1;

dEu(i) = -kl*u*Eu(i)+kli*Eo(i)-ar*alph(i)*Eu(i)*cheR...
    -arp*(1-alph(i))*Eu(i)*cheR+dr*EuR(i)+kb*EuB(i+1);

dEuR(i) = -kl*u*EuR(i)+kli*EoR(i)+ar*alph(i)*Eu(i)*cheR...
    +arp*(1-alph(i))*Eu(i)*cheR-dr*EuR(i)-kr*EuR(i);

dEo(i) = kl*u*Eu(i)-kli*Eo(i)...
    -ar*alpho(i)*Eo(i)*cheR-arp*(1-alpho(i))*Eo(i)*cheR+dr*EoR(i)+kb*EoB(i+1);

dEoR(i) = kl*u*EuR(i)-kli*EoR(i)+ar*alpho(i)*Eo(i)*cheR...
    +arp*(1-alpho(i))*Eo(i)*cheR-dr*EoR(i)-kr*EoR(i);


for i=2:n_methyl
    dEu(i) = -kl*u*Eu(i)+kli*Eo(i)-ab*alph(i)*Eu(i)*cheB...
        -abp*(1-alph(i))*Eu(i)*cheB+db*EuB(i)+kr*EuR(i-1)...
        -ar*alph(i)*Eu(i)*cheR-arp*(1-alph(i))*Eu(i)*cheR+dr*EuR(i)+kb*EuB(i+1);
    
    dEuR(i) = -kl*u*EuR(i)+kli*EoR(i)+ar*alph(i)*Eu(i)*cheR...
        +arp*(1-alph(i))*Eu(i)*cheR-dr*EuR(i)-kr*EuR(i);
    
    dEuB(i) = -kl*u*EuB(i)+kli*EoB(i)+ab*alph(i)*Eu(i)*cheB+...
        abp*(1-alph(i))*Eu(i)*cheB-db*EuB(i)-kb*EuB(i);
    
    dEo(i) = kl*u*Eu(i)-kli*Eo(i)-ab*alpho(i)*Eo(i)*cheB...
        -abp*(1-alpho(i))*Eo(i)*cheB+db*EoB(i)+kr*EoR(i-1)...
        -ar*alpho(i)*Eo(i)*cheR-arp*(1-alpho(i))*Eo(i)*cheR+dr*EoR(i)+kb*EoB(i+1);
    
    dEoR(i) = kl*u*EuR(i)-kli*EoR(i)+ar*alpho(i)*Eo(i)*cheR...
        +arp*(1-alpho(i))*Eo(i)*cheR-dr*EoR(i)-kr*EoR(i);
    
    dEoB(i) = kl*u*EuB(i)-kli*EoB(i)+ab*alpho(i)*Eo(i)*cheB+...
        abp*(1-alpho(i))*Eo(i)*cheB-db*EoB(i)-kb*EoB(i);
end

% Last iteration -> no methylation terms
i=n_methyl+1;

dEu(i) = -kl*u*Eu(i)+kli*Eo(i)-ab*alph(i)*Eu(i)*cheB...
    -abp*(1-alph(i))*Eu(i)*cheB+db*EuB(i)+kr*EuR(i-1);

dEuB(i) = -kl*u*EuB(i)+kli*EoB(i)+ab*alph(i)*Eu(i)*cheB+...
    abp*(1-alph(i))*Eu(i)*cheB-db*EuB(i)-kb*EuB(i);

dEo(i) = kl*u*Eu(i)-kli*Eo(i)-ab*alpho(i)*Eo(i)*cheB...
    -abp*(1-alpho(i))*Eo(i)*cheB+db*EoB(i)+kr*EoR(i-1);

%dEoB(i) = kl*u*EuB(i)-kli*EoB(i)+ab*alpho(i)*Eo(i)*cheB+...
%    abp*(1-alpho(i))*Eo(i)*cheB-db*EoB(i)-kb*EoB(i);

% Put state derivatives in the output vector
dx=[];

i=1;
dx=[dx; dEu(i); dEo(i); dEuR(i); dEoR(i)];

for i=0:n_methyl-2
    dx=[dx; dEu(i+2); dEo(i+2); dEuR(i+2); dEoR(i+2); dEuB(i+2); dEoB(i+2)];
end

i=n_methyl-1;
dx=[dx; dEu(i+2); dEo(i+2); dEuB(i+2)]; %; dEoB(i+2)];
